<?php
/**
 * Plugin Name: NextGEN Gallery Extension Import From Media Library
 * Description: An extension to NextGEN Gallery that let's you import media library to gallery.
 * Author: Reigel Gallarde
 * Author URI: http://reigelgallarde.me
 * Version: 0.3
 * Tested up to: 4.1
 *
 *
 * License: GNU General Public License v3.0
 * License URI: http://www.gnu.org/licenses/gpl-3.0.html
 *
 * @package     
 * @author      Reigel Gallarde
 * @category    Plugin
 * @license     http://www.gnu.org/licenses/gpl-3.0.html GNU General Public License v3.0
 */
 
 
 /**
 * Exit if accessed directly
 **/
if ( ! defined( 'ABSPATH' ) ) exit;
/**
 * Check if NextGEN Gallery by Photocrati is active
 **/
include_once( ABSPATH . 'wp-admin/includes/plugin.php' );
if ( ! is_plugin_active('nextgen-gallery/nggallery.php') ) {
    return;
}

if (! class_exists('NextGENGalleryExtensionImportFromMediaLibrary')) :

class NextGENGalleryExtensionImportFromMediaLibrary {

	public function __construct(){
		add_action('plugins_loaded',array( $this, 'init'));
	}
	
	public function init(){
		add_action( 'admin_footer', array( $this, 'admin_footer'),11 );
		add_action( 'admin_enqueue_scripts', array( $this, 'load_custom_wp_admin_style') );
		add_action( 'wp_ajax_ngg_gallery_import', array( $this, 'ngg_gallery_import') );
	}
	
	public function load_custom_wp_admin_style() {
		$currentScreen = $_REQUEST['page'];
		if( $currentScreen == "ngg_addgallery" ) {
			wp_enqueue_media();
			wp_enqueue_script('jquery-ui');
			wp_enqueue_script('jquery-ui-progressbar');
		}
	}

	public function admin_footer() {
		$currentScreen = $_REQUEST['page'];
		if( $currentScreen == "ngg_addgallery" ) {
			
			global $wpdb;
			
			$gallery_list = $wpdb->get_results("
			SELECT *
			FROM ".$wpdb->prefix."ngg_gallery
			ORDER BY gid");
			
			$gallery_options = '<option value="">Create a new gallery</option>';
			foreach ($gallery_list as $gallery) {
				$gallery_options .= '<option value="'.$gallery->gid.'">'.$gallery->name.'</option>';
			}
			$select_gallery = '<select id="ngg_gallery_list" name="ngg_gallery">'.$gallery_options.'</select>';
			
			if (wp_script_is( 'jquery', 'done' )) {
				
				//<iframe frameborder="0" src="< ?php echo get_upload_iframe_src('media'); ? >" style="width: 100%%; height: 400px;"> </iframe>
				?>
				<style>
					.list_images {
						margin: 0 -6px;
					}
					.list_images li{
						display:inline-block;
						margin:10px;
					}
					.list_images li span{
						display:block;
						font-size: 11px;
					}
					.list_images li img{
						-webkit-box-shadow: 0 0 0 1px #fff,0 0 0 3px #ccc;
						box-shadow: 0 0 0 1px #fff,0 0 0 3px #ccc;
					}
					.list_images li.start img{
						-webkit-box-shadow: 0 0 0 1px #6067F0,0 0 0 3px #009DFD;
						box-shadow: 0 0 0 1px #6067F0,0 0 0 3px #009DFD;
					}
					.list_images li.done img{
						-webkit-box-shadow: 0 0 0 1px #00982A,0 0 0 3px #00EA41;
						box-shadow: 0 0 0 1px #00982A,0 0 0 3px #00EA41;
					}
					#progressbar {
						border:0;
					}
					#progressbar .ui-progressbar-value.transition {
						-webkit-transition: width 1s; /* For Safari 3.1 to 6.0 */
						transition: width 1s;
					}
					.media-frame-title {
						top:0;
						display: block;
					}
				</style>
				<script>
					jQuery(function($){
						accordion  = '<h3 class="accordion_tab" id="import_gallery"><a href="#">Import Media Gallery</a></h3>';
						accordion += '<div id="import_gallery_content">\
											<input type="hidden" id="selected_images" />\
											<a href="#" id="upload_image_button" class="plupload_button plupload_add">Open Media Libary</a>\
											<?php echo $select_gallery; ?>\
											<input type="text" name="ngg_gallery_name" id="ngg_gallery_name" />\
											<a href="#" id="import_images_button" class="plupload_button plupload_disabled">Start Import</a>\
											<div id="progressbar" style="display:none"><span class="progress-label">Getting ready...</span><span class="prog_counter"></span></div>\
											<div class="images">\
												<ul class="list_images"></ul>\
											</div>\
											<input type="checkbox" id="keep_location" name="keep_location" value="on">\
											<label for="import_keep_location"> Keep images in original location. <span style="font-size: 85%">Caution: If you keep images in the original folder and later delete the gallery, \
											the images in that folder might be deleted depending on your settings. Also this is <strong>Limited</strong> to same folder or gallery path that you can see in a gallery in Manage Galleries</span></label>\
									  </div>';
						$(accordion).prependTo('#nextgen_admin_accordion');
						
						$('#nextgen_admin_accordion').on('change','#ngg_gallery_list',function(){
							$(this).siblings('#ngg_gallery_name').toggle($(this).val() == "");
						});
						
						var ngg_import, attachment_obj, proccessing = false, ajax_proccess_counter = 0, ajax_ngg = false;
						
						// Gallery button...
						$('#import_gallery_content').on('click', '#upload_image_button', function (event) {
							
							if (proccessing) {alert('Please wait for it to finish..'); return false;}
							
							$upload_image_button = $(this);

							event.preventDefault();

							// If the media frame already exists, reopen it.
							if (ngg_import) {
								ngg_import.open();
								return;
							}

							// Create the media frame.
							ngg_import = wp.media.frames.ngg_import = wp.media({
								title: 'Select Images to import',
								button: {
									text: 'Continue...',
								},
								id: 'ngg_import-frame',
								multiple: true,          
								frame:      'select',
								library: { // remove these to show all
									type: 'image', // specific mime
								}

							});

							// When an image is selected, run a callback.
							ngg_import.on('select', function () {
								// We set multiple to false so only get one image from the uploader
								attachment = ngg_import.state().get('selection').toJSON();
								//console.log(attachment);
								attachment_ids = [];
								li = '';
								attachment.forEach(function(img){
									if (img.id != null) {
										attachment_ids.push(img.id);
										imgurl = img.url;
										if('sizes' in img) {
											if ('thumbnail' in img.sizes){
												imgurl = img.sizes.thumbnail.url;
											}
										} else {	
										}
										li += '<li id="ngg_img'+img.id+'"><img src="'+imgurl+'" width="50px" /><span>Pending</span></li>';
									}
								});
								$('#selected_images').val(attachment_ids.join());
								$upload_image_button.text(function(){
									return (attachment_ids.length>0)?'Edit Selection':'Open Media Library';
								});
								$('.images .list_images').empty().append(li);
								$('#import_images_button').toggleClass('plupload_disabled',attachment_ids.length<1)
								attachment_obj = attachment;
								progressLabel.text('Getting ready...')
								progressbar.show().find('.ui-progressbar-value').empty().append('<div class="ui-progressbar-overlay"></div>');
							});
							
							// on open, select last selection...
							ngg_import.on('open', function () {
								
								var selection = ngg_import.state().get('selection');
								ids = $('#selected_images').val();
								if (ids != '') {
									ids = ids.split(',');
									ids.forEach(function(id) {
										attachment = wp.media.attachment(id);
										selection.add( attachment ? [ attachment ] : [] );
									});
								}
								ngg_import.$el.find('.media-sidebar').remove();
								ngg_import.$el.find('.media-frame-toolbar .media-toolbar-secondary').html('<br />(Press and hold ctrl key for multiple selection)');
								ngg_import.$el.find('.attachments-browser .attachments,.attachments-browser .media-toolbar').css({'right':0,'margin-right':0});
								ngg_import.$el.find('.media-menu a').css({'width':'100%'});
							});

							// Finally, open the modal
							ngg_import.open();
						});
						
						// Import button
						$('#import_gallery_content').on('click', '#import_images_button',function(e){
							e.preventDefault();
							ids = $('#selected_images').val();
							gallery_id = parseInt($('#ngg_gallery_list').val(),10);
							if (isNaN(gallery_id) && !$('#ngg_gallery_name').val().isAlphaNumeric()) {
								alert('Please type in gallery name with no spaces or choose a gallery...');
								return false;
							}
							
							if ((ids != '') && !proccessing) {
								ajax_proccess_counter = 0;
								prepare(ids);
							}
						});
						
						String.prototype.isAlphaNumeric = function() {
						  var regExp = /^[\w.-_]+$/;
						  return ((this.match(regExp) !== null) && !((this == '..') || (this == '.')));
						};
						// progress bar..
						
						var progressbar = $( "#progressbar" ),
						  progressLabel = $( ".progress-label" );
						makeProgressbar();
						
						function makeProgressbar(){
							progressbar.progressbar({
								value: false,
								change: function() {
									progressLabel.text( parseInt(progressbar.progressbar( "value" ),10) + "%" );
								},
								complete: function() {
									progressLabel.text( "Complete!" );
									proccess_complete();
								}
							});
						}
					 
						function progress(inc) {
							if (inc == null){inc=0;}
							var val = progressbar.progressbar( "value" ) || 0;
							progressbar.progressbar( "value", val + inc );
						}
						function prepare(ids) {
							ids = ids.split(',');
							if (ids.length <1){alert('Nothing to import.');}
							proccess_start();
							proccess(ids[ajax_proccess_counter++],ids);//ajax_proccess_counter
						}
						function proccess(id,ids,g_id){
							var prg = Math.ceil(100/ids.length);
							if (isNaN(g_id)){
								g_id = $('#import_gallery_content #ngg_gallery_list').val();
							}
							
							if (!isNaN(id)) {
								$data = {
									'action': 'ngg_gallery_import',
									'ngg_gallery': g_id,
									'ngg_gallery_name': $('#import_gallery_content #ngg_gallery_name').val(),
									'keep_location' : $('#import_gallery_content #keep_location').is(":checked") ? 'yes' : 'no',
									'ngg_check': '<?php echo wp_create_nonce( "ngg_check" ); ?>',
									'author_id': '<?php  echo get_current_user_id(); ?>',
									'attachment_id': id
								}
								$settings = {
									beforeSend : function(jqXHR,settings){
										proccessing = true;
										$(this).addClass('start').find('span').text('Proccessing');
									},
									complete : function(jqXHR) {
										if (jqXHR !== -1) {
											result = $.parseJSON(jqXHR.responseText);
											//console.log(result);
											if (result.error) {
												
												proccess_complete();
												alert(result.error);
												return;
											}
											if(result.new_gallery){
												$('<option selected value="'+result.new_gallery['id']+'">'+result.new_gallery['name']+'</option>')
												.appendTo('#ngg_gallery_list');
												$('#ngg_gallery_name').fadeOut().val('');
											}
											$(this).addClass('done').removeClass('start').find('span').text('done');
											progress(prg);
											//console.log(ajax_proccess_counter + ' : '+ ids.length)
											if (ajax_proccess_counter >= ids.length){
												proccess_complete();
											} else {
												proccess(ids[ajax_proccess_counter++],ids,result.ngg_gallery);
											}
										}
									},
									type : 'POST',
									context : $('#ngg_img'+id),
									data: $data,
									dataType : 'json'
								}
								$.ajax(ajaxurl, $settings);
							}
						}
						function proccess_start(){
							ajax_proccess_counter = 0;
							$('#import_images_button').addClass('plupload_disabled');
							$('#upload_image_button').text('Proccessing...').removeClass('plupload_add');
							progressbar.progressbar( "value", 0).find('.ui-progressbar-value').addClass('transition');
							progressbar.find('.ui-progressbar-value').empty();
						}
						function proccess_complete(){
							proccessing = false;
							$('#selected_images').val('');
							$('#upload_image_button').text('Open Media Library').addClass('plupload_add');
							$('#progressbar .ui-progressbar-value').removeClass('transition');
						}

					});
				</script>
				<?php
			}

		}
		
	}
	
	public function ngg_gallery_import(){
		// security check...
		check_ajax_referer( 'ngg_check', 'ngg_check' );
		
		//echo json_encode(array('error'=>file_exists('/home/ifgwzavl/public_html/wp-content/uploads/2015/01/Drinks.jpg')));
		//die();
		$data['post'] = $_POST;
		$data['success'] = 1;
		
		global $wpdb;
		// User definable constants
		if (!defined('NGG_IMPORT_ROOT')) {
			$path = WP_CONTENT_DIR;
			define('NGG_IMPORT_ROOT', $path);
		}
		$data['keep_location'] = sanitize_text_field($_POST['keep_location']);
		$data['ngg_path'] = str_replace('/wp-content','',NGG_IMPORT_ROOT);
		$data['id'] = sanitize_text_field($_POST['attachment_id']);
		$image_path = get_post_meta( $data['id'], '_wp_attached_file', true);
		$data['filename'] = basename($image_path);
		$image_path = str_replace('/','\\',$image_path);
		$wp_upload_dir = wp_upload_dir();
		$fimage_path = sprintf('%s/%s', $wp_upload_dir['basedir'] , $image_path);
		$fimage_path = str_replace('/','\\',$fimage_path);
		$data['fromdir'] = str_replace($data['filename'],'',$fimage_path);
		$data['ngg_gallery'] = sanitize_text_field($_POST['ngg_gallery']);
		$data['ngg_gallery_name'] = sanitize_text_field($_POST['ngg_gallery_name']);
		$data['author_id'] = sanitize_text_field($_POST['author_id']);
		if (!is_numeric($data['id'])) {
			echo json_encode(array('error'=>'Invalid Media Image ID!'));
			die();
		}
		
		$ds = DIRECTORY_SEPARATOR;
		if (empty($data['ngg_gallery'])) {
			// user wants to create a new gallery...
			if ($data['keep_location'] !== 'yes') {
				$gallery_folder = sprintf('%s/gallery/%s',NGG_IMPORT_ROOT, $data['ngg_gallery_name']);
				$gallery_path = $ds . 'wp-content'. $ds .'gallery' . $ds .$data['ngg_gallery_name'];
			} else {
				$gallery_folder = $data['fromdir'];
			}
			if (!empty($data['ngg_gallery_name']) && !file_exists($gallery_folder) && (mkdir($gallery_folder, 0777, true))) {
				// gallery folder successfully created... 
				// so let's add the info to the gallery table...
				// let's also make a thumbs folder
				mkdir($gallery_folder.'/thumbs', 0777, true);
				
			} elseif (!file_exists($gallery_folder)) {
				echo json_encode(array('error'=>'Gallery can not be created!'));
				die();
			}
			if (file_exists($gallery_folder)) {
				
				$wpdb->ngg_gallery = $wpdb->prefix.'ngg_gallery';
				$wpdb->query( $wpdb->prepare( 
					"
						INSERT INTO $wpdb->ngg_gallery
						( name, slug, path, title, author, pageid, previewpic, extras_post_id)
						VALUES ( '%s', '%s', '%s', '%s', '%d', '%d', '%d', '%d' )
					", 
						
						$data['ngg_gallery_name'], 
						$data['ngg_gallery_name'], 
						$gallery_path, 
						$data['ngg_gallery_name'], 
						$data['author_id'],
						0,
						0,
						0
				) );
				$data['ngg_gallery'] = $wpdb->insert_id;
				$data['new_gallery'] = array('id'=>$data['ngg_gallery'],'name'=>$data['ngg_gallery_name']);
				
			}
		}
		
		// let's the gallery path...
		$gallery = $wpdb->get_row($wpdb->prepare("
		SELECT *
		FROM ".$wpdb->prefix."ngg_gallery
		WHERE gid = %d
		ORDER BY gid",$data['ngg_gallery']));
		$data['gallery'] = $gallery;
		
		// copy image to NextGen gallery folder
		if ($data['keep_location'] !== 'yes') {
			if (strpos($data['gallery']->path,$data['ngg_path']) === 0) {
				$copy_image_path = $data['gallery']->path.'/';
			} else {
				$copy_image_path = sprintf('%s%s/',$data['ngg_path'],$data['gallery']->path);
			}
			$data['filename'] = $this->copy_image($data['fromdir'], $data['filename'], $copy_image_path,true);
			// make thumbnail
			if (strpos($data['gallery']->path,$data['ngg_path']) === 0) {
				$thumb_image_path = $data['gallery']->path.'/';
			} else {
				$thumb_image_path = sprintf('%s%s/',$data['ngg_path'],$data['gallery']->path);
			}
			$this->makeThumbnails($thumb_image_path, $data['filename'], '');
		} else {
			$copy_image_path = $data['fromdir'];
			$data['filename'] = $this->copy_image($data['fromdir'], $data['filename'], $copy_image_path,true);
			$this->makeThumbnails($copy_image_path, $data['filename'], '');
		}
		
		// after all that let's save image info to the database...
		$img_post = get_post($data['id'], ARRAY_A);
		$img_file_info = array(
			'image_slug' 			=> preg_replace('/\\.[^.\\s]{3,4}$/', '', $data['filename']),
			'post_id' 				=> $data['id'],
			'galleryid'				=> $data['ngg_gallery'],
			'filename'				=> $data['filename'],
			'description'			=> $img_post['post_content'],
			'alttext'				=> $img_post['post_title'],
			'imagedate'				=> $img_post['post_date'],
			'exclude'				=> 0,
			'sortorder'				=> 0,
			'meta_data'				=> $this->build_meta(str_replace('\\','/',$copy_image_path.'/'. $data['filename'])),
			'extras_post_id'		=> 0,
			'updated_at'			=> NULL	
		);
		
		$this->add_image_to_nggdb($img_file_info);
		
		// return results as json.
		header( "Content-Type: application/json" );
		echo json_encode($data);
		die();
	}
	
	private function add_image_to_nggdb($img){
		global $wpdb;
		
		$wpdb->ngg_pictures = $wpdb->prefix.'ngg_pictures';
		
		$wpdb->query( $wpdb->prepare( 
			"
				INSERT INTO $wpdb->ngg_pictures
				( image_slug, post_id, galleryid, filename, description, alttext, imagedate, exclude, sortorder, meta_data, extras_post_id, updated_at )
				VALUES ( '%s', '%d', '%d', '%s', '%s', '%s', '%s', '%d', '%d', '%s', '%d', '%d' )
			", 
				$img['image_slug'],
				$img['post_id'],
				$img['galleryid'],
				$img['filename'],
				$img['description'],
				$img['alttext'],
				$img['imagedate'],
				$img['exclude'],
				$img['sortorder'],
				$img['meta_data'],
				$img['extras_post_id'],
				$img['updated_at']	
				
		) );
	}
	
	// copy the image to gallery folder, then return the filename...
	private function copy_image($fromdir, $filename, $todir, $bkup = false){
		$from = rtrim($fromdir,'\\') . '\\' . $filename;
		$to = $this->test_image(rtrim($todir,'\\') . '\\' . $filename);
		$from = str_replace('\\','/',$from);
		$to = str_replace('\\','/',$to);
		//echo json_encode(array('error'=>file_exists($from)));
		//die();
		if (!copy($from, $to)) {
			$errors= error_get_last();
			echo json_encode(array('error'=>'COPY ERROR: ' . $errors['type'] . ' - ' . $errors['message']));
			die();
		}
		if ($bkup) {
			if (!copy($from, $to.'_backup')) {
				$errors = error_get_last();
				$data['warning']= 'COPY ERROR: ' . $errors['type'] . ' - ' . $errors['message'];
			}
		}
		return basename($to);
	}
	
	private function makeThumbnails($updir, $filename){
		$updir = str_replace('\\','/',$updir);
		$thumbnail_width = 120;
		$thumbnail_height = 90;
		$thumb_beforeword = "thumbs/thumbs_";
		$arr_image_details = getimagesize("$updir" . "$filename"); // pass id to thumb name
		$original_width = $arr_image_details[0];
		$original_height = $arr_image_details[1];
		if ($original_width < $original_height) {
			$new_width = $thumbnail_width;
			$new_height = intval($original_height * $new_width / $original_width);
		} else {
			$new_height = $thumbnail_height;
			$new_width = intval($original_width * $new_height / $original_height);
		}
		$dest_x = intval(($thumbnail_width - $new_width) / 2);
		$dest_y = intval(($thumbnail_height - $new_height) / 2);
		if ($arr_image_details[2] == 1) {
			$imgt = "ImageGIF";
			$imgcreatefrom = "ImageCreateFromGIF";
			$compression = 80;
		}
		if ($arr_image_details[2] == 2) {
			$imgt = "ImageJPEG";
			$imgcreatefrom = "ImageCreateFromJPEG";
			$compression = 80;
		}
		if ($arr_image_details[2] == 3) {
			$imgt = "ImagePNG";
			$imgcreatefrom = "ImageCreateFromPNG";
			$compression = 8;
		}
		if ($imgt) {
			$old_image = $imgcreatefrom("$updir" . "$filename");
			$new_image = imagecreatetruecolor($thumbnail_width, $thumbnail_height);
			imagecopyresampled($new_image, $old_image, $dest_x, $dest_y, 0, 0, $new_width, $new_height, $original_width, $original_height);
			$to = $this->test_image("$updir" . "$thumb_beforeword" . "$filename");
			$imgt($new_image, $to ,$compression);
		}
	}
	
	private function build_meta($img_path){
		$filename = basename($img_path);
		list($width, $height, $type, $attr) = getimagesize($img_path);
		$array = array(
			'width'				=> $width,
            'height'				=> $height,
            'full'				=> array(
									'filename'	=> $filename,
									'width'		=> $width,
									'height'		=> $height
								),                          
            'thumbnail'			=> array(
									'width'		=> 120,
									'height'		=> 90,
									'filename'	=>'thumbs_'.$filename,
									'generated'	=>'0.62859300 1377499003' //fudging this bit because I have no clue what it is. 
								),
            'aperature'			=> false,          
            'credit'				=> false,
            'camera'				=> false,
            'caption'				=> false,
            'created_timestamp'	=> false,
            'copyright'			=> false,
            'focal_length'		=> false,
            'iso'					=> false,
            'shutter_speed'		=> false,
            'flash'				=> false,
            'title'				=> false,
            'keywords'			=> NULL,
            'saved'				=> false
        );
        $array = serialize($array);
        return $array;
	}
	
	
	// helper functions
	
	private function str_replace_last( $search , $replace , $str ) {
		if( ( $pos = strrpos( $str , $search ) ) !== false ) {
			$search_length  = strlen( $search );
			$str    = substr_replace( $str , $replace , $pos , $search_length );
		}
		return $str;
	}
	// rename image that exists
	private function test_image($img){
		if (file_exists($img)) {
			return $this->test_image($this->str_replace_last('.','-1.',$img));
		} else {
			return $img;
		}
	}
}
new NextGENGalleryExtensionImportFromMediaLibrary();
endif;